/*
 * UPnP Browser Library.
 *
 * Copyright (C) 2006 INdT
 * @author Gustavo Sverzut Barbieri <gustavo.barbieri@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __UPNP_BROWSER_H__
#define __UPNP_BROWSER_H__

#include <glib.h>
#include <cybergarage/upnp/ccontrolpoint.h>
#include <libupnpbrowser/upnp-cds-object.h>

G_BEGIN_DECLS
/***********************************************************************
 * UPnP Browser
 **********************************************************************/
typedef enum
{
    UPNP_BROWSER_DEVICE_STATUS_ADDED   = CgUpnpDeviceStatusAdded,
    UPNP_BROWSER_DEVICE_STATUS_UPDATED = CgUpnpDeviceStatusUpdated,
    UPNP_BROWSER_DEVICE_STATUS_INVALID = CgUpnpDeviceStatusInvalid,
    UPNP_BROWSER_DEVICE_STATUS_REMOVED = CgUpnpDeviceStatusRemoved
} upnp_browser_device_status_t;

typedef struct upnp_browser upnp_browser_t;

typedef void (*upnp_browser_main_loop_addler_t) (
    void *func,
    void* data);

typedef void (*upnp_browser_device_listener_callback_t) (
    upnp_browser_t *browser,
    const char *udn,
    upnp_browser_device_status_t status,
    gpointer user_data);

struct upnp_browser
{
    gboolean started;
    CgUpnpControlPoint *control_point;

    /* device listener */
    gpointer device_listener_user_data;
    GFreeFunc device_listener_user_data_free_func;
    upnp_browser_device_listener_callback_t device_listener;
};

#define UPNP_BROWSER_ERROR upnp_browser_error_quark ()
typedef enum
{
    UPNP_BROWSER_ERROR_INVALID_ACTION                              = 401,
    UPNP_BROWSER_ERROR_INVALID_ARGS                                = 402,
    UPNP_BROWSER_ERROR_INVALID_VAR                                 = 404,
    UPNP_BROWSER_ERROR_ACTION_FAILED                               = 501,
    UPNP_BROWSER_ERROR_ARGUMENT_VALUE_INVALID                      = 600,
    UPNP_BROWSER_ERROR_ARGUMENT_VALUE_OUT_OF_RANGE                 = 601,
    UPNP_BROWSER_ERROR_OPTIONAL_ACTION_NOT_IMPLEMENTED             = 602,
    UPNP_BROWSER_ERROR_OUT_OF_MEMORY                               = 603,
    UPNP_BROWSER_ERROR_HUMAN_INTERVENTION_REQUIRED                 = 604,
    UPNP_BROWSER_ERROR_STRING_ARGUMENT_TOO_LONG                    = 605,
    UPNP_BROWSER_ERROR_ACTION_NOT_AUTHORIZED                       = 606,
    UPNP_BROWSER_ERROR_SIGNATURE_FAILED                            = 607,
    UPNP_BROWSER_ERROR_SIGNATURE_MISSING                           = 608,
    UPNP_BROWSER_ERROR_NOT_ENCRYPTED                               = 609,
    UPNP_BROWSER_ERROR_INVALID_SEQUENCE                            = 610,
    UPNP_BROWSER_ERROR_INVALID_CONTROL_URL                         = 611,
    UPNP_BROWSER_ERROR_NO_SUCH_SESSION                             = 612,
    UPNP_BROWSER_ERROR_NO_SUCH_OBJECT                              = 701,
    UPNP_BROWSER_ERROR_INVALID_CURRENT_TAG_VALUE                   = 702,
    UPNP_BROWSER_ERROR_INVALID_NEW_TAG_VALUE                       = 703,
    UPNP_BROWSER_ERROR_REQUIRED_TAG                                = 704,
    UPNP_BROWSER_ERROR_READ_ONLY_TAG                               = 705,
    UPNP_BROWSER_ERROR_PARAMETER_MISMATCH                          = 706,
    UPNP_BROWSER_ERROR_UNSUPPORTED_SEARCH_CRITERIA                 = 708,
    UPNP_BROWSER_ERROR_UNSUPPORTED_SORT_CRITERIA                   = 709,
    UPNP_BROWSER_ERROR_NO_SUCH_CONTAINER                           = 710,
    UPNP_BROWSER_ERROR_RESTRICTED_OBJECT                           = 711,
    UPNP_BROWSER_ERROR_BAD_METADATA                                = 712,
    UPNP_BROWSER_ERROR_RESTRICTED_PARENT_OBJECT                    = 713,
    UPNP_BROWSER_ERROR_NO_SUCH_SOURCE_RESOURCE                     = 714,
    UPNP_BROWSER_ERROR_SOURCE_RESOURCE_ACCESS_DENIED               = 715,
    UPNP_BROWSER_ERROR_TRANSFER_BUSY                               = 716,
    UPNP_BROWSER_ERROR_NO_SUCH_FILE_TRANSFER                       = 717,
    UPNP_BROWSER_ERROR_NO_SUCH_DESTINATION_RESOURCE                = 718,
    UPNP_BROWSER_ERROR_DESTINATION_RESOURCE_ACCESS_DENIED          = 719,
    UPNP_BROWSER_ERROR_CANNOT_PROCESS_THE_REQUEST                  = 720
} upnp_browser_error_code_t;


void               upnp_browser_init                 (void);
upnp_browser_t    *upnp_browser_new                  (void) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
void               upnp_browser_destroy              (upnp_browser_t *browser);

int                upnp_browser_get_device_notification_fd (upnp_browser_t *browser);
void               upnp_browser_remove_device_notification_fd (upnp_browser_t *browser, int fd_read);
void               upnp_browser_remove_all_notification_fd (upnp_browser_t *browser);

void               upnp_browser_set_device_listener  (upnp_browser_t *browser,
                                                      upnp_browser_device_listener_callback_t func,
                                                      gpointer user_data,
                                                      GFreeFunc user_data_free);

GSList            *upnp_browser_list_folder_complete (upnp_browser_t *browser,
                                                      const gchar *udn,
                                                      const gchar *parent_id,
                                                      guint starting_index,
                                                      gint max_count,
                                                      const gchar *property_filter,
                                                      const gchar *sort_criteria,
                                                      GError **error);

GSList            *upnp_browser_list_folder          (upnp_browser_t *browser,
                                                      const gchar *udn,
                                                      const gchar *folder_id,
                                                      GError **error) G_GNUC_WARN_UNUSED_RESULT;

gboolean           upnp_browser_start                (upnp_browser_t *browser);
gboolean           upnp_browser_stop                 (upnp_browser_t *browser);
gchar             *upnp_browser_get_device_name      (upnp_browser_t *browser,
                                                      const gchar *udn) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
GSList            *upnp_browser_get_device_services  (upnp_browser_t *browser, const gchar *udn);

GSList            *upnp_browser_get_devices          (upnp_browser_t *browser) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
upnp_cds_object_t *upnp_browser_get_object           (upnp_browser_t *browser,
                                                      const gchar *udn,
                                                      const gchar *id,
                                                      GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
gchar           **upnp_browser_get_sort_caps         (upnp_browser_t *browser,
                                                      const gchar *udn,
                                                      GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;

void               upnp_browser_quit                 (void);
GQuark             upnp_browser_error_quark          (void) G_GNUC_CONST;

G_END_DECLS
#endif /* __UPNP_BROWSER_H__ */
