/*
 * UPnP Browser Library.
 *
 * Copyright (C) 2006 INdT
 * @author Gustavo Sverzut Barbieri <gustavo.barbieri@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __UPNP_CDS_OBJECT_H__
#define __UPNP_CDS_OBJECT_H__

#include <glib.h>

G_BEGIN_DECLS

/**
 * As ContentDirectory 1.0 - 2.4.2 ``Base Properties''.
 */
typedef struct upnp_cds_object
{
    enum
    {
        UPNP_CDS_OBJECT_TYPE_ITEM = 1,
        UPNP_CDS_OBJECT_TYPE_CONTAINER
    } type;

    /* Required */
    char *klass;
    char *parent_id; /**< root is "-1" */
#define UPNP_CDS_OBJECT_PARENT_ID_ROOT "-1"

    char *id;
    char *title;
    char *creator;
    gboolean restricted;

    GHashTable *info; /**< values are GSList of upnp_cds_info_t */

    union
    {
        struct
        {
            char *referred_id;
        } item;
        struct
        {
            int child_count;
            gboolean searchable;
        } container;
    } spec;
} upnp_cds_object_t;


typedef struct upnp_cds_info
{
    GHashTable *attributes;
    gchar *contents;
} upnp_cds_info_t;


void                   upnp_cds_object_destroy             (upnp_cds_object_t *object);


char                  *upnp_cds_object_str                 (const upnp_cds_object_t *obj) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;

const GSList          *upnp_cds_object_get_info_list       (const upnp_cds_object_t *object,
                                                            const gchar *key);
const upnp_cds_info_t *upnp_cds_object_get_info_struct     (const upnp_cds_object_t *object,
                                                            const gchar *key);
const gchar           *upnp_cds_object_get_info            (const upnp_cds_object_t *object,
                                                            const gchar *key);
int                    upnp_cds_object_get_info_as_int     (const upnp_cds_object_t *object,
                                                            const gchar *key);
gboolean               upnp_cds_object_get_info_as_boolean (const upnp_cds_object_t *object,
                                                            const gchar *key);
const gchar           *upnp_cds_info_get_attribute         (const upnp_cds_info_t *info,
                                                            const gchar *name);
GSList                *upnp_cds_object_get_res_by_protocol (const upnp_cds_object_t *object,
                                                            const gchar *protocol,
                                                            const gchar *network,
                                                            const gchar *mime_type,
                                                            const gchar *extra_info) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;

G_END_DECLS
#endif /* __UPNP_CDS_OBJECT_H__ */
